unit VoiceCommandsControlU;

//Uses the high level Microsoft Voice Commands ActiveX to simplify coding

interface

uses
  Speech,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, AppEvnts, Menus, StdCtrls, ComCtrls, OleCtrls, HSRLib_TLB;

type
  TfrmVoiceCommandsControl = class(TForm)
    Vcommand: TVcommand;
    GroupBox1: TGroupBox;
    lblMinVU: TLabel;
    lblMaxVU: TLabel;
    lblVU: TLabel;
    ProgressBar: TProgressBar;
    lstProgress: TListBox;
    MainMenu1: TMainMenu;
    mniFile: TMenuItem;
    mniFileExit: TMenuItem;
    mniOptions: TMenuItem;
    mniOptionsClearLog: TMenuItem;
    mniOptionsFormColour: TMenuItem;
    mniWindow: TMenuItem;
    mniWindowMinimise: TMenuItem;
    mniWindowMaximise: TMenuItem;
    mniWindowRestore: TMenuItem;
    mniHelp: TMenuItem;
    mniHelpAbout: TMenuItem;
    dlgColor: TColorDialog;
    StatusBar1: TStatusBar;
    ApplicationEvents1: TApplicationEvents;
    chkEnabled: TCheckBox;
    chkAwake: TCheckBox;
    btnAbout: TButton;
    btnGeneral: TButton;
    btnLexicon: TButton;
    btnTrainGeneral: TButton;
    btnTrainMic: TButton;
    procedure FormCreate(Sender: TObject);
    procedure VcommandAttribChanged(Sender: TObject; attrib: Integer);
    procedure VcommandCommandOther(Sender: TObject; const CmdName,
      command: WideString);
    procedure VcommandCommandRecognize(Sender: TObject; ID: Integer;
      const CmdName: WideString; Flags: Integer; const Action: WideString;
      NumLists: Integer; const ListValues, command: WideString);
    procedure VcommandCommandStart(Sender: TObject);
    procedure VcommandInterference(Sender: TObject);
    procedure VcommandMenuActivate(Sender: TObject;
      const CmdName: WideString; bActive: Integer);
    procedure VcommandUtteranceBegin(Sender: TObject);
    procedure VcommandUtteranceEnd(Sender: TObject);
    procedure VcommandVUMeter(Sender: TObject; Level: Integer);
    procedure mniFileExitClick(Sender: TObject);
    procedure mniHelpAboutClick(Sender: TObject);
    procedure mniOptionsClearLogClick(Sender: TObject);
    procedure mniOptionsFormColourClick(Sender: TObject);
    procedure mniWindowMaximiseClick(Sender: TObject);
    procedure mniWindowMinimiseClick(Sender: TObject);
    procedure mniWindowRestoreClick(Sender: TObject);
    procedure ApplicationEvents1Activate(Sender: TObject);
    procedure ApplicationEvents1Deactivate(Sender: TObject);
    procedure chkEnabledClick(Sender: TObject);
    procedure chkAwakeClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnGeneralClick(Sender: TObject);
    procedure btnLexiconClick(Sender: TObject);
    procedure btnTrainGeneralClick(Sender: TObject);
    procedure btnTrainMicClick(Sender: TObject);
  private
    CmdMenu: Integer;
    procedure CreateCommandMenu;
  public
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; const Args: array of const); overload;
  end;

var
  frmVoiceCommandsControl: TfrmVoiceCommandsControl;

implementation

{$R *.dfm}

uses
  ComObj;

{$ifndef CONDITIONALEXPRESSIONS}
function BoolToStr(B: Boolean; UseBoolStrs: Boolean = False): string;
const
  cSimpleBoolStrs: array[Boolean] of String = ('0', '-1');
  cBoolStrs: array[Boolean] of String = ('False', 'True');
begin
  if UseBoolStrs then
    Result := cBoolStrs[B]
  else
    Result := cSimpleBoolStrs[B];
end;
{$endif}

procedure TfrmVoiceCommandsControl.Log(const Msg: String);
begin
  if not Assigned(lstProgress) then
    Exit;
  lstProgress.Items.Add(Msg);
  lstProgress.ItemIndex := lstProgress.Items.Count - 1
end;

procedure TfrmVoiceCommandsControl.Log(const Msg: String; const Args: array of const);
begin
  Log(Format(Msg, Args))
end;

procedure TfrmVoiceCommandsControl.FormCreate(Sender: TObject);
begin
  Vcommand.Initialized := Integer(True);
  chkEnabled.Checked := Bool(Vcommand.Enabled);
  chkAwake.Checked := Bool(Vcommand.AwakeState);
  CreateCommandMenu;
end;

procedure TfrmVoiceCommandsControl.CreateCommandMenu;

  procedure AddMenuCommands(Item: TMenuItem; const ParentPath: String);
  var
    I: Integer;
    Path: String;
  begin
    Path := ParentPath + StripHotKey(Item.Caption);
    //Recurse through subitems, if any
    if (Item.Count > 0) then
      for I := 0 to Item.Count - 1 do
        AddMenuCommands(Item.Items[I], Path + #32)
    //Otherwise add this item, if appropriate
    else
      if (Item.Caption <> '') and (Item.Caption <> '-') then
        //Must pass a non-blank strings for all WideString parameters
        Vcommand.AddCommand(CmdMenu, Item.Command, Path,
          Item.Hint, 'Menu', VCMDCMD_CANTRENAME, ' ');
  end;

begin
  CmdMenu := VCommand.MenuCreate[
    ExtractFileName(Application.ExeName), 'Main', VCMDMC_CREATE_TEMP];
  AddMenuCommands(Menu.Items, '');
  Vcommand.Activate(CmdMenu);
end;

procedure TfrmVoiceCommandsControl.VcommandAttribChanged(Sender: TObject;
  attrib: Integer);
var
  S: String;
begin
  if attrib <> 0 then
  begin
    if attrib and IVCNSAC_AUTOGAINENABLE > 0 then
      S := 'automatic gain, ';
    if attrib and IVCNSAC_ENABLED > 0 then
    begin
      S := S + 'enabled, ';
      chkEnabled.Checked := Bool(Vcommand.Enabled);
    end;
    if attrib and IVCNSAC_AWAKE > 0 then
    begin
      S := S + 'awake, ';
      chkAwake.Checked := Bool(Vcommand.AwakeState);
    end;
    if attrib and IVCNSAC_DEVICE > 0 then
      S := S + 'audio device, ';
    if attrib and IVCNSAC_MICROPHONE > 0 then
      S := S + 'current microphone, ';
    if attrib and IVCNSAC_SPEAKER > 0 then
      S := S + 'speaker, ';
    if attrib and IVCNSAC_SRMODE > 0 then
      S := S + 'SR mode, ';
    if attrib and IVCNSAC_THRESHOLD > 0 then
      S := S + 'threshold, ';
    if attrib and IVCNSAC_ORIGINAPP > 0 then
      S := S + 'from this app';
  end
  else
    S := 'none';
  Log('OnAttribChanged: ' + S)
end;

procedure TfrmVoiceCommandsControl.VcommandCommandOther(Sender: TObject; const CmdName,
  command: WideString);
begin
  Log('OnCommandOther: CmdName = %s, Command = %s', [CmdName, Command])
end;

procedure TfrmVoiceCommandsControl.VcommandCommandRecognize(Sender: TObject;
  ID: Integer; const CmdName: WideString; Flags: Integer;
  const Action: WideString; NumLists: Integer; const ListValues,
  command: WideString);
begin
  Log('OnCommandRecognize: CmdName = %s, Command = %s, ID = %d', [CmdName, command, ID]);
  Perform(WM_COMMAND, ID, 0)
end;

procedure TfrmVoiceCommandsControl.VcommandCommandStart(Sender: TObject);
begin
  Log('OnCommandStart')
end;

procedure TfrmVoiceCommandsControl.VcommandInterference(Sender: TObject);
begin
  Log('OnInterference')
end;

procedure TfrmVoiceCommandsControl.VcommandMenuActivate(Sender: TObject;
  const CmdName: WideString; bActive: Integer);
begin
  Log('OnMenuActivate: CmdName = %s, Active: %s', [CmdName, BoolToStr(Bool(bActive), True)])
end;

procedure TfrmVoiceCommandsControl.VcommandUtteranceBegin(Sender: TObject);
begin
  Log('UtteranceBegin')
end;

procedure TfrmVoiceCommandsControl.VcommandUtteranceEnd(Sender: TObject);
begin
  Log('OnUtteranceEnd')
end;

procedure TfrmVoiceCommandsControl.VcommandVUMeter(Sender: TObject; Level: Integer);
begin
  ProgressBar.Position := Level;
  lblVU.Caption := IntToStr(Level);
end;

procedure TfrmVoiceCommandsControl.mniFileExitClick(Sender: TObject);
begin
  Close
end;

procedure TfrmVoiceCommandsControl.mniOptionsClearLogClick(
  Sender: TObject);
begin
  lstProgress.Clear
end;

procedure TfrmVoiceCommandsControl.mniOptionsFormColourClick(Sender: TObject);
begin
  dlgColor.Color := Color;
  if dlgColor.Execute then
    Color := dlgColor.Color
end;

procedure TfrmVoiceCommandsControl.mniWindowMinimiseClick(
  Sender: TObject);
begin
  Application.Minimize
end;

procedure TfrmVoiceCommandsControl.mniWindowMaximiseClick(
  Sender: TObject);
begin
  WindowState := wsMaximized
end;

procedure TfrmVoiceCommandsControl.mniWindowRestoreClick(
  Sender: TObject);
begin
  Application.Restore;
  WindowState := wsNormal
end;

procedure TfrmVoiceCommandsControl.mniHelpAboutClick(Sender: TObject);
begin
  MessageDlg(
    'Voice Commands ActiveX Control Demo'#13#13'Copyright  2002, Brian Long',
    mtInformation, [mbOk], 0)
end;

procedure TfrmVoiceCommandsControl.ApplicationEvents1Activate(
  Sender: TObject);
begin
  Vcommand.Activate(CmdMenu);
end;

procedure TfrmVoiceCommandsControl.ApplicationEvents1Deactivate(
  Sender: TObject);
begin
  Vcommand.Deactivate(CmdMenu);
end;

procedure TfrmVoiceCommandsControl.chkEnabledClick(Sender: TObject);
begin
  Vcommand.Enabled := Integer(chkEnabled.Checked);
end;

procedure TfrmVoiceCommandsControl.chkAwakeClick(Sender: TObject);
begin
  Vcommand.AwakeState := Integer(chkAwake.Checked);
end;

procedure TfrmVoiceCommandsControl.btnAboutClick(Sender: TObject);
begin
  Vcommand.AboutDlg(Handle, '');
end;

procedure TfrmVoiceCommandsControl.btnGeneralClick(Sender: TObject);
begin
  Vcommand.GeneralDlg(Handle, '');
end;

procedure TfrmVoiceCommandsControl.btnLexiconClick(Sender: TObject);
begin
  Vcommand.LexiconDlg(Handle, '');
end;

procedure TfrmVoiceCommandsControl.btnTrainGeneralClick(Sender: TObject);
begin
  Vcommand.TrainGeneralDlg(Handle, '');
end;

procedure TfrmVoiceCommandsControl.btnTrainMicClick(Sender: TObject);
begin
  Vcommand.TrainMicDlg(Handle, '');
end;

end.
