unit VoiceCommandAutoVarU;

//Uses the VCmdAuto Automation object, with late bound access through a Variant

interface

uses
  Speech,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Menus, ComCtrls;

type
  TfrmVoiceCommandAutomation = class(TForm)
    lstProgress: TListBox;
    MainMenu1: TMainMenu;
    StatusBar1: TStatusBar;
    mniFile: TMenuItem;
    mniFileExit: TMenuItem;
    mniOptions: TMenuItem;
    mniOptionsClearLog: TMenuItem;
    mniOptionsFormColour: TMenuItem;
    dlgColor: TColorDialog;
    mniWindow: TMenuItem;
    mniWindowMinimise: TMenuItem;
    mniWindowMaximise: TMenuItem;
    mniWindowRestore: TMenuItem;
    mniHelp: TMenuItem;
    mniHelpAbout: TMenuItem;
    chkAwake: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure mniFileExitClick(Sender: TObject);
    procedure mniOptionsClearLogClick(Sender: TObject);
    procedure mniOptionsFormColourClick(Sender: TObject);
    procedure mniWindowMinimiseClick(Sender: TObject);
    procedure mniWindowMaximiseClick(Sender: TObject);
    procedure mniWindowRestoreClick(Sender: TObject);
    procedure mniHelpAboutClick(Sender: TObject);
    procedure chkAwakeClick(Sender: TObject);
  private
    VCmd, VMenu: Variant;
    procedure CreateCommandMenu;
  public
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; const Args: array of const); overload;
  end;

var
  frmVoiceCommandAutomation: TfrmVoiceCommandAutomation;

implementation

{$R *.dfm}

uses
  ComObj;

procedure TfrmVoiceCommandAutomation.Log(const Msg: String);
begin
  if not Assigned(lstProgress) then
    Exit;
  lstProgress.Items.Add(Msg);
  lstProgress.ItemIndex := lstProgress.Items.Count - 1
end;

procedure TfrmVoiceCommandAutomation.Log(const Msg: String; const Args: array of const);
begin
  Log(Format(Msg, Args))
end;

procedure TfrmVoiceCommandAutomation.FormCreate(Sender: TObject);
begin
  SendMessage(lstProgress.Handle, LB_SETHORIZONTALEXTENT, Width, 0);
  VCmd := CreateOleObject('Speech.VoiceCommand');
  VCmd.Register('');
  VCmd.Callback := 'VoiceCommandAutoVar.ListenCallback';
  CreateCommandMenu;
  chkAwake.Checked := VCmd.Awake;
end;

procedure TfrmVoiceCommandAutomation.CreateCommandMenu;

  procedure AddMenuCommands(Item: TMenuItem; const ParentPath: String);
  var
    I: Integer;
    Path: String;
  begin
    Path := ParentPath + StripHotKey(Item.Caption);
    //Recurse through subitems, if any
    if (Item.Count > 0) then
      for I := 0 to Item.Count - 1 do
        AddMenuCommands(Item.Items[I], Path + #32)
    //Otherwise add this item, if appropriate
    else
      if (Item.Caption <> '') and (Item.Caption <> '-') then
        VMenu.Add(Item.Command, Path, ' ', Item.Hint);
  end;

begin
  //Must pass a non-blank strings for all WideString parameters
  VMenu := VCmd.MenuCreate(ExtractFileName(Application.ExeName),
    'Main Menu', LANG_ENGLISH, ' ', vcmdmc_CREATE_TEMP);
  AddMenuCommands(Menu.Items, '');
  VMenu.hWndMenu := Handle;
  VMenu.Active := True;
end;

procedure TfrmVoiceCommandAutomation.mniFileExitClick(Sender: TObject);
begin
  Close
end;

procedure TfrmVoiceCommandAutomation.mniOptionsClearLogClick(
  Sender: TObject);
begin
  lstProgress.Clear
end;

procedure TfrmVoiceCommandAutomation.mniOptionsFormColourClick(Sender: TObject);
begin
  dlgColor.Color := Color;
  if dlgColor.Execute then
    Color := dlgColor.Color
end;

procedure TfrmVoiceCommandAutomation.mniWindowMinimiseClick(
  Sender: TObject);
begin
  Application.Minimize
end;

procedure TfrmVoiceCommandAutomation.mniWindowMaximiseClick(
  Sender: TObject);
begin
  WindowState := wsMaximized
end;

procedure TfrmVoiceCommandAutomation.mniWindowRestoreClick(
  Sender: TObject);
begin
  Application.Restore;
  WindowState := wsNormal
end;

procedure TfrmVoiceCommandAutomation.mniHelpAboutClick(Sender: TObject);
begin
  MessageDlg(
    'Voice Command Late Bound Automation Demo'#13#13'Copyright  2002, Brian Long',
    mtInformation, [mbOk], 0)
end;

procedure TfrmVoiceCommandAutomation.chkAwakeClick(Sender: TObject);
begin
  VCmd.Awake := chkAwake.Checked;
end;

end.
